﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PostSharp.Laos;
using System.Reflection;

namespace Logica.Kbn.PostSharpAspect
{
    [Serializable()]
    public class IntBoolFunctionCacheAspect : OnMethodBoundaryAspect
    {
        // dict of function names (!) and cache dictionary
        private static Dictionary<MethodBase, Dictionary<int, bool>>
            cache = new Dictionary<MethodBase, Dictionary<int, bool>>();

        public override void OnEntry(MethodExecutionEventArgs eventArgs)
        {
            // create fct dict if not exists
            if (!cache.ContainsKey(eventArgs.Method))
                cache.Add(eventArgs.Method, new Dictionary<int, bool>());

            // get fct-dict and fct-parameter
            Dictionary<int, bool> fctCache = cache[eventArgs.Method];
            int param = (int)eventArgs.GetReadOnlyArgumentArray()[0];

            // lookup calculated result
            if (fctCache.ContainsKey(param))
            {
                eventArgs.ReturnValue = fctCache[param];
                eventArgs.FlowBehavior = FlowBehavior.Return;
            }
        }

        public override void OnSuccess(MethodExecutionEventArgs eventArgs)
        {
            MethodBase fctName = eventArgs.Method;
            Dictionary<int, bool> fctCache = cache[fctName];
            int param = (int)eventArgs.GetReadOnlyArgumentArray()[0];
            bool retVal = (bool)eventArgs.ReturnValue;

            fctCache.Add(param, retVal);
        }
    }
}
